@@ -14,12 +14,12 @@ from tg.support.middlewares import SessionMiddleware, CacheMiddleware
 from tg.support.middlewares import StaticsMiddleware, SeekableRequestBodyMiddleware, \
     DBSessionRemoverMiddleware
 from tg.support.registry import RegistryManager
-from tg.support.converters import asbool, asint
+from tg.support.converters import asbool, asint, aslist
 from tg.request_local import config as reqlocal_config
 
 import tg
 from tg.configuration.utils import coerce_config
-from tg.util import Bunch, get_partial_dict, DottedFileNameFinder, call_controller
+from tg.util import Bunch, get_partial_dict, DottedFileNameFinder
 from tg.configuration import milestones
 from tg.configuration.utils import TGConfigError
 
@@ -36,7 +36,7 @@ class DispatchingConfigWrapper(DictMixin):
     """Wrapper for the Dispatching configuration.
 
     Simple wrapper for the DispatchingConfig object that provides attribute
-    style access to the Pylons config dictionary.
+    style access to the config dictionary.
 
     This class works by proxying all attribute and dictionary access to
     the underlying DispatchingConfig config object, which is an application local
@@ -44,10 +44,6 @@ class DispatchingConfigWrapper(DictMixin):
     in the same process simultaneously, but to always get the right
     config data for the application that's requesting them.
 
-    Sites, with seeking to maximize needs may prefer to use the Pylons
-    config stacked object proxy directly, using just dictionary style
-    access, particularly whenever config is checked on a per-request basis.
-
     """
 
     def __init__(self, dict_to_wrap):
@@ -124,6 +120,29 @@ reqlocal_config.push_process_config(deepcopy(defaults))
 config = DispatchingConfigWrapper(reqlocal_config)
 
 
+def call_controller(tg_config, controller, remainder, params):
+    return controller(*remainder, **params)
+
+
+class _DeprecatedControllerWrapper(object):
+    def __init__(self, controller_wrapper, config, next_wrapper):
+        # Backward compatible old-way of configuring controller wrappers
+        warnings.warn("Controller wrapper will now accept the configuration"
+                      "as parameter when called instead of receiving it as"
+                      "a constructor parameter, please refer to the documentation"
+                      "to update your controller wrappers",
+                      DeprecationWarning, stacklevel=2)
+
+        def _adapted_next_wrapper(controller, remainder, params):
+            return next_wrapper(tg.config._current_obj(),
+                                controller, remainder, params)
+
+        self.wrapper = controller_wrapper(config, _adapted_next_wrapper)
+
+    def __call__(self, config, controller, remainder, params):
+        return self.wrapper(controller, remainder, params)
+
+
 class AppConfig(Bunch):
     """Class to store application configuration.
 
@@ -189,7 +208,6 @@ class AppConfig(Bunch):
         self.call_on_shutdown = []
         self.controller_caller = call_controller
         self.controller_wrappers = []
-        self.dedicated_controller_wrappers = {}
         self.application_wrappers = []
         self.application_wrappers_dependencies = {False: [],
                                                   None: []}
@@ -396,6 +414,20 @@ class AppConfig(Bunch):
 
         conf['tg.errorware'] = errorware
 
+        slowreqsware = coerce_config(conf, 'trace_slowreqs.', {'smtp_use_tls': asbool,
+                                                               'dump_request_size': asint,
+                                                               'dump_request': asbool,
+                                                               'dump_local_frames': asbool,
+                                                               'dump_local_frames_count': asint,
+                                                               'enable': asbool,
+                                                               'interval': asint,
+                                                               'exclude': aslist})
+        slowreqsware.setdefault('error_subject_prefix', 'Slow Request: ')
+        slowreqsware.setdefault('error_message', 'A request is taking too much time')
+        for erroropt in errorware:
+            slowreqsware.setdefault(erroropt, errorware[erroropt])
+        conf['tg.slowreqs'] = slowreqsware
+
         # Copy in some defaults
         if 'cache_dir' in conf:
             conf.setdefault('beaker.session.data_dir', os.path.join(conf['cache_dir'], 'sessions'))
@@ -590,7 +622,7 @@ class AppConfig(Bunch):
         datastore_options.pop('port', None)
 
         datastore = create_ming_datastore(config['ming.url'], config.get('ming.db', ''), **datastore_options)
-        config['pylons.app_globals'].ming_datastore = datastore
+        config['tg.app_globals'].ming_datastore = datastore
         self.package.model.init_model(datastore)
 
     def setup_sqlalchemy(self):
@@ -687,21 +719,12 @@ class AppConfig(Bunch):
 
         controller_caller = base_controller_caller
         for wrapper in self.get('controller_wrappers', []):
-            controller_caller = wrapper(self, controller_caller)
+            try:
+                controller_caller = wrapper(controller_caller)
+            except TypeError:
+                controller_caller = _DeprecatedControllerWrapper(wrapper, self, controller_caller)
         config['controller_caller'] = controller_caller
 
-        dedicated_wrappers = config.get('dedicated_controller_wrappers', {})
-        for wrapped_controller in dedicated_wrappers:
-            controller_caller = base_controller_caller
-            wrappers = dedicated_wrappers[wrapped_controller]
-            # Apply custom wrappers for controller
-            for wrapper in wrappers:
-                controller_caller = wrapper(self, controller_caller)
-            # Apply generic wrappers for application
-            for wrapper in self.get('controller_wrappers', []):
-                controller_caller = wrapper(self, controller_caller)
-            dedicated_wrappers[wrapped_controller] = controller_caller
-
     def _setup_renderers(self):
         for renderer in self.renderers[:]:
             setup = getattr(self, 'setup_%s_renderer'%renderer, None)
@@ -793,6 +816,10 @@ class AppConfig(Bunch):
 
         return app
 
+    def add_slowreqs_middleware(self, global_conf, app):
+        from tg.error import SlowReqsReporter
+        return SlowReqsReporter(app, global_conf, **config['tg.slowreqs'])
+
     def add_debugger_middleware(self, global_conf, app):
         from tg.error import ErrorHandler
         return ErrorHandler(app, global_conf)
@@ -1164,13 +1191,14 @@ class AppConfig(Bunch):
                 full_stack = False
 
             if asbool(full_stack):
+                # This should never be true for internal nested apps
                 if (self.auth_backend is None
                         and 401 not in self.handle_status_codes):
                     # If there's no auth backend configured which traps 401
                     # responses we redirect those responses to a nicely
                     # formatted error page
                     self.handle_status_codes.append(401)
-                # This should never be true for internal nested apps
+                app = self.add_slowreqs_middleware(global_conf, app)
                 app = self.add_error_middleware(global_conf, app)
 
             # Establish the registry for this application
