@@ -10,17 +10,18 @@ needed to support these decorators.
 """
 import copy
 import warnings
-from decorator import decorator
-
+import time
 from webob.exc import HTTPUnauthorized, HTTPMethodNotAllowed, HTTPMovedPermanently
+from tg.support import NoDefault
 from tg.support.paginate import Page
 from tg.configuration import config
+from tg.configuration.app_config import _DeprecatedControllerWrapper, call_controller
 from tg.controllers.util import abort, redirect
 from tg import tmpl_context, request, response
 from tg.util import partial, Bunch
 from tg.configuration.sqla.balanced_session import force_request_engine
 from tg.flash import flash
-from tg.caching import beaker_cache, cached_property
+from tg.caching import beaker_cache, cached_property, _cached_call
 from tg.predicates import NotAuthorizedError
 from tg._compat import default_im_func, unicode_text
 from webob.acceptparse import Accept
@@ -30,11 +31,22 @@ import tg
 import logging
 log = logging.getLogger(__name__)
 
+
+def _decorated_controller_caller(tg_config, controller, remainder, params):
+    try:
+        application_controller_caller = tg_config['controller_caller']
+    except KeyError:
+        application_controller_caller = call_controller
+
+    return application_controller_caller(tg_config, controller, remainder, params)
+
+
 class Decoration(object):
     """ Simple class to support 'simple registration' type decorators
     """
     def __init__(self, controller):
         self.controller = controller
+        self.controller_caller = _decorated_controller_caller
         self._expositions = []
         self.engines = {}
         self.engines_keys = []
@@ -43,7 +55,7 @@ class Decoration(object):
         self.render_custom_format = None
         self.validation = None
         self.inherit = False
-        self.requirement = None
+        self.requirements = []
         self.hooks = dict(before_validate=[],
                           before_call=[],
                           before_render=[],
@@ -60,12 +72,15 @@ class Decoration(object):
             dec = func.decoration = cls(func)
         return dec
 
-    def _register_exposition(self, exposition, inherit=False):
+    def _register_exposition(self, exposition, inherit=False, before=False):
         """Register an exposition for later application"""
 
         # We need to store a reference to the exposition
         # so that we can merge them when inheritance is performed
-        self._expositions.append(exposition)
+        if before:
+            self._expositions.insert(0, exposition)
+        else:
+            self._expositions.append(exposition)
 
         if inherit:
             # if at least one exposition is in inherit mode
@@ -83,6 +98,15 @@ class Decoration(object):
             except IndexError:
                 break
 
+    @property
+    def requirement(self):  # pragma: no cover
+        warnings.warn("Decoration.requirement is deprecated, "
+                      "please use 'requirements' instead", DeprecationWarning)
+        try:
+            return self.requirements[0]
+        except IndexError:
+            return None
+
     @property
     def exposed(self):
         return bool(self.engines) or bool(self.custom_engines)
@@ -94,8 +118,8 @@ class Decoration(object):
         self.custom_engines = dict(tuple(deco.custom_engines.items()) + tuple(self.custom_engines.items()))
 
         # This merges yet to register template engines
-        for exposition in deco._expositions:
-            self._register_exposition(exposition._clone(self.controller))
+        for exposition in reversed(deco._expositions):
+            self._register_exposition(exposition._clone(self.controller), before=True)
 
         # inherit all the parent hooks
         # parent hooks before current hooks so that they get called before
@@ -260,6 +284,17 @@ class Decoration(object):
         """
         self.hooks.setdefault(hook_name, []).append(func)
 
+    def _register_requirement(self, requirement):
+        self._register_hook('before_call', requirement._check_authorization)
+        self.requirements.append(requirement)
+
+    def _register_controller_wrapper(self, wrapper):
+        try:
+            self.controller_caller = wrapper(self.controller_caller)
+        except TypeError:
+            self.controller_caller = _DeprecatedControllerWrapper(wrapper, tg.config,
+                                                                  self.controller_caller)
+
 
 class _hook_decorator(object):
     """Superclass for all the specific TG2 hook validators.
@@ -752,47 +787,44 @@ def with_trailing_slash(remainder, params):
         redirect(request.url+'/', redirect_with=HTTPMovedPermanently)
 
 
-#{ Authorization decorators
-class _BaseProtectionDecorator(object):
-    default_denial_handler = None
-
-    def __init__(self, predicate, denial_handler=None, smart_denial=False):
-        """Verify that the predicate is met.
-
-        :param predicate: An object with a check_authorization(environ) method which
-            must raise a tg.predicates.NotAuthorizedError if not met.
-        :param denial_handler: The callable to be run if authorization is
-            denied (overrides :attr:`default_denial_handler` if defined).
-
-        If called, ``denial_handler`` will be passed a positional argument
-        which represents a message on why authorization was denied.
-
-        """
-        self.predicate = predicate
-        self.denial_handler = denial_handler or self.default_denial_handler
-        self.smart_denial = smart_denial
-
-class require(_BaseProtectionDecorator):
+class require(object):
     """
-    TurboGears-specific action protector.
+    Decorator that checks if the specified predicate it met, if it isn't
+    it calls the denial_handler to prevent access to the decorated method.
 
     The default authorization denial handler of this protector will flash
     the message of the unmet predicate with ``warning`` or ``error`` as the
     flash status if the HTTP status code is 401 or 403, respectively.
 
-    See :class:`allow_only` for controller-wide authorization.
+    :param predicate: An object with a check_authorization(environ) method which
+        must raise a tg.predicates.NotAuthorizedError if not met.
+    :param denial_handler: The callable to be run if authorization is
+        denied (overrides :attr:`default_denial_handler` if defined).
+    :param smart_denial: A list of response types for which to trigger
+        the smart denial, which will act as an API providing a pass-through
+        :func:`tg.controllers.util.abort`.
+        If ``True``, ``('application/json', 'text/xml')`` will be used.
+
+    If called, ``denial_handler`` will be passed a positional argument
+    which represents a message on why authorization was denied.
+
+    Use ``allow_only`` property of ``TGController`` for controller-wide authorization.
 
     """
-    def __call__(self, action_):
-        # Even though @require is not Decoration based
-        # provide a requirement attribute to decoration to
-        # let the user inspect the method requirements
-        deco = Decoration.get_decoration(action_)
-        deco.requirement = self
+    def __init__(self, predicate, denial_handler=None, smart_denial=False):
+        self.predicate = predicate
+        self.denial_handler = denial_handler or self.default_denial_handler
 
-        return decorator(self.wrap_action, action_)
+        if smart_denial is True:
+            smart_denial = ('application/json', 'text/xml')
+        self.smart_denial = smart_denial
 
-    def wrap_action(self, action_, *args, **kwargs):
+    def __call__(self, func):
+        deco = Decoration.get_decoration(func)
+        deco._register_requirement(self)
+        return func
+
+    def _check_authorization(self, *args, **kwargs):
         req = request._current_obj()
 
         try:
@@ -807,17 +839,26 @@ class require(_BaseProtectionDecorator):
                 code = 401
             response.status = code
             return self.denial_handler(reason)
-        return action_(*args, **kwargs)
 
     def default_denial_handler(self, reason):
         """Authorization denial handler for protectors."""
-        status = 'warning' if response.status_int == 401 else 'error'
-        if not self.smart_denial:
+        passthrough_abort = False
+
+        if self.smart_denial:
+            response_type = response.content_type or request.response_type
+            if response_type in self.smart_denial:
+                # It's an API response, use a pass-through abort
+                passthrough_abort = True
+                if response_type == 'application/json':
+                    passthrough_abort = 'json'
+
+        if passthrough_abort is False:
+            # Plain HTML page
+            status = 'warning' if response.status_int == 401 else 'error'
             flash(reason, status=status)
-        else:
-            if response.content_type not in ['application/json', 'text/xml']:
-                flash(reason, status=status)
-        abort(response.status_int, reason)
+
+        abort(response.status_int, reason, passthrough=passthrough_abort)
+
 
 class with_engine(object):
     """
@@ -859,4 +900,84 @@ class with_engine(object):
         decoration = Decoration.get_decoration(func)
         decoration._register_hook('before_validate', self.before_validate)
         return func
-#}
+
+
+class cached(object):
+    """
+    Decorator to cache the controller, if you also want to cache
+    template remember to return ``tg_cache`` option from the controller.
+
+    The following parameters are accepted:
+
+    ``key`` - Specifies the controller parameters used to generate the cache key.
+        NoDefault - Uses function name and all request parameters as the key (default)
+        
+        None - No variable key, uses only function name as key
+        
+        string - Use function name and only "key" parameter
+        
+        list - Use function name and all parameters listed
+    ``expire``
+        Time in seconds before cache expires, or the string "never".
+        Defaults to "never"
+    ``type``
+        Type of cache to use: dbm, memory, file, memcached, or None for
+        Beaker's default
+    ``cache_headers``
+        A tuple of header names indicating response headers that
+        will also be cached.
+    ``invalidate_on_startup``
+        If True, the cache will be invalidated each time the application
+        starts or is restarted.
+    ``cache_response``
+        Determines whether the response at the time the cache is used
+        should be cached or not, defaults to True.
+
+        .. note::
+            When cache_response is set to False, the cache_headers
+            argument is ignored as none of the response is cached.
+    """
+    def __init__(self, key=NoDefault, expire="never", type=None,
+                 query_args=None,  # Backward compatibility, actually ignored
+                 cache_headers=('content-type', 'content-length'),
+                 invalidate_on_startup=False, cache_response=True,
+                 **b_kwargs):
+        self.key = key
+        self.expire = expire
+        self.type = type
+        self.cache_headers = cache_headers
+        self.invalidate_on_startup = invalidate_on_startup
+        self.cache_response = cache_response
+        self.beaker_options = b_kwargs
+
+    def __call__(self, func):
+        decoration = Decoration.get_decoration(func)
+
+        def controller_wrapper(__, next_caller):
+            if self.invalidate_on_startup:
+                starttime = time.time()
+            else:
+                starttime = None
+
+            def cached_call_controller(controller, remainder, params):
+                if self.key:
+                    key_dict = tg.request.args_params
+                    if self.key != NoDefault:
+                        if isinstance(self.key, (list, tuple)):
+                            key_dict = dict((k, key_dict[k]) for k in key_dict)
+                        else:
+                            key_dict = {self.key: key_dict[self.key]}
+                else:
+                    key_dict = {}
+
+                return _cached_call(next_caller, (controller, remainder, params), {},
+                                    key_func=func, key_dict=key_dict,
+                                    expire=self.expire, type=self.type,
+                                    starttime=starttime, cache_headers=self.cache_headers,
+                                    cache_response=self.cache_response,
+                                    cache_extra_args=self.beaker_options)
+
+            return cached_call_controller
+
+        decoration._register_controller_wrapper(controller_wrapper)
+        return func
