@@ -17,9 +17,15 @@ except ImportError: #pragma: no cover
         """FormEncode Schema"""
 
 def _navigate_tw2form_children(w):
-    if getattr(w, 'id', None):
+    if getattr(w, 'compound_key', None):
+        # If we have a compound_key it's a leaf widget with form values
         yield w
     else:
+        child = getattr(w, 'child', None)
+        if child:
+            # Widgets with "child" don't have children, but their child has
+            w = child
+
         for c in getattr(w, 'children', []):
             for cc in _navigate_tw2form_children(c):
                 yield cc
@@ -43,4 +49,4 @@ class TGValidationError(Exception):
     def __str__(self):
         return self.msg
 
-validation_errors = (_Tw2ValidationError, _FormEncodeValidationError, TGValidationError)
\ No newline at end of file
+validation_errors = (_Tw2ValidationError, _FormEncodeValidationError, TGValidationError)
